@@ -5,6 +5,13 @@
 #ifndef RE2_REGEXP_H_
 #define RE2_REGEXP_H_
 
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wpragmas"
+#pragma GCC diagnostic ignored "-Wpedantic"
+#pragma GCC diagnostic ignored "-Wgnu-anonymous-struct"
+#pragma GCC diagnostic ignored "-Wnested-anon-types"
+#pragma GCC diagnostic ignored "-Wsign-conversion"
+
 // --- SPONSORED LINK --------------------------------------------------
 // If you want to use this library for regular expression matching,
 // you should use re2/re2.h, which provides a class RE2 that
@@ -86,7 +93,6 @@
 // form accessible to clients, so that client code can analyze the
 // parsed regular expressions.
 
-#include <stddef.h>
 #include <stdint.h>
 #include <map>
 #include <set>
@@ -209,7 +215,7 @@ class RegexpStatus {
 
  private:
   RegexpStatusCode code_;  // Kind of error
-  StringPiece error_arg_;  // Piece of regexp containing syntax error.
+  StringPiece error_arg_{};  // Piece of regexp containing syntax error.
   std::string* tmp_;       // Temporary storage, possibly where error_arg_ is.
 
   RegexpStatus(const RegexpStatus&) = delete;
@@ -596,4 +602,5 @@ inline Regexp::ParseFlags operator~(Regexp::ParseFlags a) {
 
 }  // namespace re2
 
+#pragma GCC diagnostic pop
 #endif  // RE2_REGEXP_H_
