@@ -17,74 +17,74 @@
  * GNU General Public License for more details.
  */
 
-#include <mata/inter-aut.hh>
-#include <mata/util.hh>
+#include "mata/parser/inter-aut.hh"
 
-namespace
-{
-    bool has_atmost_one_auto_naming(const Mata::IntermediateAut& aut)
+namespace {
+    bool has_atmost_one_auto_naming(const mata::IntermediateAut& aut)
     {
-        return !(!(aut.node_naming == Mata::IntermediateAut::Naming::AUTO &&
-              aut.symbol_naming == Mata::IntermediateAut::Naming::AUTO) &&
-                 (aut.state_naming == Mata::IntermediateAut::AUTO));
+        return !(!(aut.node_naming == mata::IntermediateAut::Naming::AUTO &&
+                   aut.symbol_naming == mata::IntermediateAut::Naming::AUTO) &&
+                 (aut.state_naming == mata::IntermediateAut::Naming::AUTO));
     }
 
     bool is_logical_operator(char ch)
     {
-        return (Mata::Util::haskey(std::set<char>{'&', '|', '!'}, ch));
+        return (mata::utils::haskey(std::set<char>{ '&', '|', '!'}, ch));
     }
 
-    Mata::IntermediateAut::Naming get_naming_type(const std::string &key)
+    mata::IntermediateAut::Naming get_naming_type(const std::string &key)
     {
         const size_t found = key.find('-');
         assert(found != std::string::npos);
         const std::string& type = key.substr(found+1, std::string::npos);
 
         if (type == "auto")
-            return Mata::IntermediateAut::Naming::AUTO;
+            return mata::IntermediateAut::Naming::AUTO;
         else if (type == "enum")
-            return Mata::IntermediateAut::Naming::ENUM;
+            return mata::IntermediateAut::Naming::ENUM;
         else if (type == "marked")
-            return Mata::IntermediateAut::Naming::MARKED;
+            return mata::IntermediateAut::Naming::MARKED;
         else if (type == "chars")
-            return Mata::IntermediateAut::Naming::CHARS;
+            return mata::IntermediateAut::Naming::CHARS;
         else if (type == "utf")
-            return Mata::IntermediateAut::Naming::UTF;
+            return mata::IntermediateAut::Naming::UTF;
 
         assert(false && "Unknown naming type - a naming type should be always defined correctly otherwise it is"
                         "impossible to parse automaton correctly");
+        return {};
     }
 
-    Mata::IntermediateAut::AlphabetType get_alphabet_type(const std::string &type)
+    mata::IntermediateAut::AlphabetType get_alphabet_type(const std::string &type)
     {
         assert(type.find('-') != std::string::npos);
         const std::string& alph_type = type.substr(type.find('-')+1, std::string::npos);
 
         if (alph_type == "bits") {
-            return Mata::IntermediateAut::AlphabetType::BITVECTOR;
+            return mata::IntermediateAut::AlphabetType::BITVECTOR;
         } else if (alph_type == "explicit") {
-            return Mata::IntermediateAut::AlphabetType::EXPLICIT;
+            return mata::IntermediateAut::AlphabetType::EXPLICIT;
         } else if (alph_type == "intervals") {
-            return Mata::IntermediateAut::AlphabetType::INTERVALS;
+            return mata::IntermediateAut::AlphabetType::INTERVALS;
         }
 
         assert(false && "Unknown alphabet type - an alphabet type should be always defined correctly otherwise it is"
                         "impossible to parse automaton correctly");
+        return {};
     }
 
-    bool is_naming_marker(const Mata::IntermediateAut::Naming naming)
+    bool is_naming_marker(const mata::IntermediateAut::Naming naming)
     {
-        return naming == Mata::IntermediateAut::Naming::MARKED;
+        return naming == mata::IntermediateAut::Naming::MARKED;
     }
 
-    bool is_naming_enum(const Mata::IntermediateAut::Naming naming)
+    bool is_naming_enum(const mata::IntermediateAut::Naming naming)
     {
-        return naming == Mata::IntermediateAut::Naming::ENUM;
+        return naming == mata::IntermediateAut::Naming::ENUM;
     }
 
-    bool is_naming_auto(const Mata::IntermediateAut::Naming naming)
+    bool is_naming_auto(const mata::IntermediateAut::Naming naming)
     {
-        return naming == Mata::IntermediateAut::Naming::AUTO;
+        return naming == mata::IntermediateAut::Naming::AUTO;
     }
 
     bool contains(const std::vector<std::string> &vec, const std::string &item)
@@ -92,16 +92,18 @@ namespace
         return std::find(vec.begin(), vec.end(), item) != vec.end();
     }
 
-    bool no_operators(const std::vector<Mata::FormulaNode> nodes)
-    {
-        for (const auto& node : nodes)
-            if (node.is_operator())
+    bool no_operators(const std::vector<mata::FormulaNode>& nodes) {
+        // Refactor using all_of() when Clang adds support for it.
+        // return std::ranges::all_of(nodes, [](const mata::FormulaNode& node){ return !node.is_operator();});
+        for (const auto& node: nodes) {
+            if (node.is_operator()) {
                 return false;
-
+            }
+        }
         return true;
     }
 
-    std::string serialize_graph(const Mata::FormulaGraph& graph)
+    std::string serialize_graph(const mata::FormulaGraph& graph)
     {
         if (graph.node.is_operand())
             return graph.node.raw;
@@ -128,72 +130,70 @@ namespace
         return lhs + " " + graph.node.raw + " " + rhs;
     }
 
-    Mata::FormulaNode create_node(const Mata::IntermediateAut &mata, const std::string &token)
-    {
+    mata::FormulaNode create_node(const mata::IntermediateAut &mata, const std::string &token) {
         if (is_logical_operator(token[0])) {
             switch (token[0]) {
                 case '&':
-                    return Mata::FormulaNode{Mata::FormulaNode::Type::OPERATOR, token, token,
-                                         Mata::FormulaNode::OperatorType::AND};
+                    return { mata::FormulaNode::Type::OPERATOR, token, token,
+                    mata::FormulaNode::OperatorType::AND };
                 case '|':
-                    return Mata::FormulaNode{Mata::FormulaNode::Type::OPERATOR, token, token,
-                                             Mata::FormulaNode::OperatorType::OR};
+                    return { mata::FormulaNode::Type::OPERATOR, token, token,
+                    mata::FormulaNode::OperatorType::OR };
                 case '!':
-                    return Mata::FormulaNode{Mata::FormulaNode::Type::OPERATOR, token, token,
-                                             Mata::FormulaNode::OperatorType::NEG};
+                    return { mata::FormulaNode::Type::OPERATOR, token, token,
+                    mata::FormulaNode::OperatorType::NEG };
                 default:
                     assert(false);
             }
         } else if (token == "(") {
-            return Mata::FormulaNode{Mata::FormulaNode::Type::LEFT_PARENTHESIS, token};
+            return { mata::FormulaNode::Type::LEFT_PARENTHESIS, token};
         } else if (token == ")") {
-            return Mata::FormulaNode{Mata::FormulaNode::Type::RIGHT_PARENTHESIS, token};
+            return { mata::FormulaNode::Type::RIGHT_PARENTHESIS, token};
         } else if (token == "true") {
-            return Mata::FormulaNode{Mata::FormulaNode::Type::OPERAND, token, token,
-                                     Mata::FormulaNode::OperandType::SYMBOL};
+            return { mata::FormulaNode::Type::OPERAND, token, token,
+                                      mata::FormulaNode::OperandType::SYMBOL};
         } else if (token == "false") {
-            return Mata::FormulaNode{Mata::FormulaNode::Type::OPERAND, token, token,
-                                     Mata::FormulaNode::OperandType::SYMBOL};
+            return { mata::FormulaNode::Type::OPERAND, token, token,
+                                      mata::FormulaNode::OperandType::SYMBOL};
         } else if (is_naming_enum(mata.state_naming) && contains(mata.states_names, token)) {
-            return Mata::FormulaNode{Mata::FormulaNode::Type::OPERAND, token, token,
-                                     Mata::FormulaNode::OperandType::STATE};
+            return { mata::FormulaNode::Type::OPERAND, token, token,
+                                      mata::FormulaNode::OperandType::STATE};
         } else if (is_naming_enum(mata.node_naming) && contains(mata.nodes_names, token)) {
-            return Mata::FormulaNode{Mata::FormulaNode::Type::OPERAND, token, token,
-                                     Mata::FormulaNode::OperandType::NODE};
+            return { mata::FormulaNode::Type::OPERAND, token, token,
+                                      mata::FormulaNode::OperandType::NODE};
         } else if (is_naming_enum(mata.symbol_naming) && contains(mata.symbols_names, token)) {
-            return Mata::FormulaNode{Mata::FormulaNode::Type::OPERAND, token, token,
-                                     Mata::FormulaNode::OperandType::SYMBOL};
+            return { mata::FormulaNode::Type::OPERAND, token, token,
+                                      mata::FormulaNode::OperandType::SYMBOL};
         } else if (is_naming_marker(mata.state_naming) && token[0] == 'q') {
-            return Mata::FormulaNode{Mata::FormulaNode::Type::OPERAND, token, token.substr(1),
-                                     Mata::FormulaNode::OperandType::STATE};
+            return { mata::FormulaNode::Type::OPERAND, token, token.substr(1),
+                                      mata::FormulaNode::OperandType::STATE};
         } else if (is_naming_marker(mata.node_naming) && token[0] == 'n') {
-            return Mata::FormulaNode{Mata::FormulaNode::Type::OPERAND, token, token.substr(1),
-                                     Mata::FormulaNode::OperandType::NODE};
+            return { mata::FormulaNode::Type::OPERAND, token, token.substr(1),
+                                      mata::FormulaNode::OperandType::NODE};
         } else if (is_naming_marker(mata.symbol_naming) && token[0] == 'a') {
-            return Mata::FormulaNode{Mata::FormulaNode::Type::OPERAND, token, token.substr(1),
-                                     Mata::FormulaNode::OperandType::SYMBOL};
+            return { mata::FormulaNode::Type::OPERAND, token, token.substr(1),
+                                      mata::FormulaNode::OperandType::SYMBOL};
         } else if (is_naming_auto(mata.state_naming)) {
-            return Mata::FormulaNode{Mata::FormulaNode::Type::OPERAND, token, token,
-                                     Mata::FormulaNode::OperandType::STATE};
+            return { mata::FormulaNode::Type::OPERAND, token, token,
+                                      mata::FormulaNode::OperandType::STATE};
         } else if (is_naming_auto(mata.node_naming)) {
-            return Mata::FormulaNode{Mata::FormulaNode::Type::OPERAND, token, token,
-                                     Mata::FormulaNode::OperandType::NODE};
+            return { mata::FormulaNode::Type::OPERAND, token, token,
+                                      mata::FormulaNode::OperandType::NODE};
         } else if (is_naming_auto(mata.symbol_naming)) {
-            return Mata::FormulaNode{Mata::FormulaNode::Type::OPERAND, token, token,
-                                     Mata::FormulaNode::OperandType::SYMBOL};
+            return { mata::FormulaNode::Type::OPERAND, token, token,
+                                      mata::FormulaNode::OperandType::SYMBOL};
         }
 
         throw std::runtime_error("Unknown token " + token);
-        assert(false);
     }
 
     /**
      * Checks if @p op1 has lower precedence than @p op2. Precedence from the lowest to highest is: | < & < !
      */
-    bool lower_precedence(Mata::FormulaNode::OperatorType op1, Mata::FormulaNode::OperatorType op2) {
-        if (op1 == Mata::FormulaNode::NEG) {
+    bool lower_precedence(mata::FormulaNode::OperatorType op1, mata::FormulaNode::OperatorType op2) {
+        if (op1 == mata::FormulaNode::OperatorType::NEG) {
             return false;
-        } else if (op1 == Mata::FormulaNode::AND && op2 != Mata::FormulaNode::NEG) {
+        } else if (op1 == mata::FormulaNode::OperatorType::AND && op2 != mata::FormulaNode::OperatorType::NEG) {
             return false;
         }
 
@@ -206,53 +206,56 @@ namespace
      * @param tokens Series of tokens representing transition formula parsed from the input text
      * @return A postfix notation for input
      */
-    std::vector<Mata::FormulaNode> infix_to_postfix(
-            const Mata::IntermediateAut &aut, const std::vector<std::string> &tokens) {
-        std::vector<Mata::FormulaNode> opstack;
-        std::vector<Mata::FormulaNode> output;
+    std::vector<mata::FormulaNode> infix_to_postfix(
+            const mata::IntermediateAut &aut, const std::vector<std::string>& tokens) {
+        std::vector<mata::FormulaNode> opstack;
+        std::vector<mata::FormulaNode> output;
 
-        for (const auto& token : tokens) {
-            Mata::FormulaNode node = create_node(aut, token);
+        for (const std::string& token: tokens) {
+            mata::FormulaNode node = create_node(aut, token);
             switch (node.type) {
-                case Mata::FormulaNode::OPERAND:
-                    output.push_back(node);
+                case mata::FormulaNode::Type::OPERAND:
+                    output.emplace_back(std::move(node));
                     break;
-                case Mata::FormulaNode::LEFT_PARENTHESIS:
-                    opstack.push_back(node);
+                case mata::FormulaNode::Type::LEFT_PARENTHESIS:
+                    opstack.emplace_back(std::move(node));
                     break;
-                case Mata::FormulaNode::RIGHT_PARENTHESIS:
+                case mata::FormulaNode::Type::RIGHT_PARENTHESIS:
                     while (!opstack.back().is_leftpar()) {
-                        output.push_back(opstack.back());
+                        output.emplace_back(std::move(opstack.back()));
                         opstack.pop_back();
                     }
                     opstack.pop_back();
                     break;
-                case Mata::FormulaNode::OPERATOR:
-                    for (int j = opstack.size()-1; j >= 0; --j) {
-                        assert(!opstack[j].is_operand());
-                        if (opstack[j].is_leftpar())
+                case mata::FormulaNode::Type::OPERATOR:
+                    for (int j = static_cast<int>(opstack.size())-1; j >= 0; --j) {
+                        size_t j_size_t{ static_cast<size_t>(j) };
+                        assert(!opstack[j_size_t].is_operand());
+                        if (opstack[j_size_t].is_leftpar())
                             break;
-                        if (lower_precedence(node.operator_type, opstack[j].operator_type)) {
-                            output.push_back(opstack[j]);
+                        if (lower_precedence(node.operator_type, opstack[j_size_t].operator_type)) {
+                            output.emplace_back(std::move(opstack[j_size_t]));
                             opstack.erase(opstack.begin()+j);
                         }
                     }
-                    opstack.push_back(node);
+                    opstack.emplace_back(std::move(node));
                     break;
                 default: assert(false);
             }
         }
 
         while (!opstack.empty()) {
-            output.push_back(opstack.back());
+            output.push_back(std::move(opstack.back()));
             opstack.pop_back();
         }
 
+        #ifndef NDEBUG
         for (const auto& node : output) {
             assert(node.is_operator() || (node.name != "!" && node.name != "&" && node.name != "|"));
             assert(node.is_leftpar() || node.name != "(");
             assert(node.is_rightpar() || node.name != ")");
         }
+        #endif // #ifndef NDEBUG.
 
         return output;
     }
@@ -262,31 +265,31 @@ namespace
      * @param postfix A postfix representation of transition formula
      * @return A parsed graph
      */
-    Mata::FormulaGraph postfix_to_graph(const std::vector<Mata::FormulaNode> &postfix)
-    {
-        std::vector<Mata::FormulaGraph> opstack;
-
-        for (const auto& node : postfix) {
-            Mata::FormulaGraph gr(node);
-            switch (node.type) {
-                case Mata::FormulaNode::OPERAND:
-                    opstack.push_back(gr);
+    mata::FormulaGraph postfix_to_graph(std::vector<mata::FormulaNode> postfix) {
+            std::vector<mata::FormulaGraph> opstack{};
+            opstack.reserve(4);
+            for (mata::FormulaNode& node: postfix) {
+            mata::FormulaGraph gr(std::move(node));
+            switch (gr.node.type) {
+                case mata::FormulaNode::Type::OPERAND:
+                    opstack.emplace_back(std::move(gr));
                     break;
-                case Mata::FormulaNode::OPERATOR:
-                    switch (node.operator_type) {
-                        case Mata::FormulaNode::NEG:
+                case mata::FormulaNode::Type::OPERATOR:
+                    switch (gr.node.operator_type) {
+                        case mata::FormulaNode::OperatorType::NEG:
                             assert(!opstack.empty());
-                            gr.children.push_back(opstack.back());
+                            gr.children.emplace_back(std::move(opstack.back()));
                             opstack.pop_back();
-                            opstack.push_back(gr);
+                            opstack.emplace_back(std::move(gr));
                             break;
                         default:
                             assert(opstack.size() > 1);
-                            gr.children.push_back(opstack.back());
+                            mata::FormulaGraph second_child{ std::move(opstack.back()) };
                             opstack.pop_back();
-                            gr.children.insert(gr.children.begin(), opstack.back());
+                            gr.children.emplace_back(std::move(opstack.back()));
                             opstack.pop_back();
-                            opstack.push_back(gr);
+                            gr.children.emplace_back(std::move(second_child));
+                            opstack.emplace_back(std::move(gr));
                     }
                     break;
                 default: assert(false && "Unknown type of node");
@@ -294,9 +297,8 @@ namespace
         }
 
         assert(opstack.size() == 1);
-
-        return opstack.back();
-    }
+        return std::move(opstack[0]);
+}
 
     /**
      * Function adds disjunction operators to a postfix form when there are no operators at all.
@@ -305,7 +307,7 @@ namespace
      * @param postfix Postfix to which operators are eventually added
      * @return A postfix with eventually added operators
      */
-    std::vector<Mata::FormulaNode> add_disjunction_implicitly(const std::vector<Mata::FormulaNode> &postfix)
+    std::vector<mata::FormulaNode> add_disjunction_implicitly(std::vector<mata::FormulaNode> postfix)
     {
         if (postfix.size() == 1) // no need to add operators
             return postfix;
@@ -315,18 +317,18 @@ namespace
                 return postfix;
         }
 
-        std::vector<Mata::FormulaNode> res;
+        std::vector<mata::FormulaNode> res;
         if (postfix.size() >= 2) {
             res.push_back(postfix[0]);
             res.push_back(postfix[1]);
-            res.push_back(Mata::FormulaNode(
-                    Mata::FormulaNode::OPERATOR, "|", "|", Mata::FormulaNode::OR));
+            res.emplace_back(
+                    mata::FormulaNode::Type::OPERATOR, "|", "|", mata::FormulaNode::OperatorType::OR);
         }
 
         for (size_t i = 2; i < postfix.size(); ++i) {
             res.push_back(postfix[i]);
-            res.push_back(Mata::FormulaNode(
-                    Mata::FormulaNode::OPERATOR, "|", "|", Mata::FormulaNode::OR));
+            res.emplace_back(
+                    mata::FormulaNode::Type::OPERATOR, "|", "|", mata::FormulaNode::OperatorType::OR);
         }
 
         return res;
@@ -339,14 +341,13 @@ namespace
      * @param section A section of input MATA format
      * @return Parsed InterAutomata representing an automaton from input.
      */
-    Mata::IntermediateAut mf_to_aut(const Mata::Parser::ParsedSection &section)
-    {
-        Mata::IntermediateAut aut;
+    mata::IntermediateAut mf_to_aut(const mata::parser::ParsedSection& section) {
+        mata::IntermediateAut aut;
 
         if (section.type.find("NFA") != std::string::npos) {
-            aut.automaton_type = Mata::IntermediateAut::AutomatonType::NFA;
+            aut.automaton_type = mata::IntermediateAut::AutomatonType::NFA;
         } else if (section.type.find("AFA") != std::string::npos) {
-            aut.automaton_type = Mata::IntermediateAut::AutomatonType::AFA;
+            aut.automaton_type = mata::IntermediateAut::AutomatonType::AFA;
         }
         aut.alphabet_type = get_alphabet_type(section.type);
 
@@ -378,16 +379,17 @@ namespace
                 auto postfix = infix_to_postfix(aut, keypair.second);
                 if (no_operators(postfix)) {
                     aut.initial_enumerated = true;
-                    postfix = add_disjunction_implicitly(postfix);
+                    postfix = add_disjunction_implicitly(std::move(postfix));
                 }
-                aut.initial_formula = postfix_to_graph(postfix);
+                aut.initial_formula = postfix_to_graph(std::move(postfix));
             } else if (key.find("Final") != std::string::npos) {
                 auto postfix = infix_to_postfix(aut, keypair.second);
                 if (no_operators(postfix)) {
-                    postfix = add_disjunction_implicitly(postfix);
+                    postfix = add_disjunction_implicitly(std::move(postfix));
                     aut.final_enumerated = true;
                 }
-                aut.final_formula = postfix_to_graph(postfix);
+
+                aut.final_formula = postfix_to_graph(std::move(postfix));;
             }
         }
 
@@ -396,14 +398,14 @@ namespace
         }
 
         for (const auto& trans : section.body) {
-            Mata::IntermediateAut::parse_transition(aut,trans);
+            mata::IntermediateAut::parse_transition(aut, trans);
         }
 
         return aut;
     }
 } // anonymous
 
-size_t Mata::IntermediateAut::get_number_of_disjuncts() const
+size_t mata::IntermediateAut::get_number_of_disjuncts() const
 {
     size_t res = 0;
 
@@ -415,12 +417,12 @@ size_t Mata::IntermediateAut::get_number_of_disjuncts() const
         while (!stack.empty()) {
             const FormulaGraph *gr = stack.back();
             stack.pop_back();
-            if (gr->node.is_operator() && gr->node.operator_type == FormulaNode::OR)
+            if (gr->node.is_operator() && gr->node.operator_type == FormulaNode::OperatorType::OR)
                 trans_disjuncts++;
             for (const auto &ch: gr->children)
                 stack.push_back(&ch);
         }
-        res += std::max(trans_disjuncts, (size_t) 1);
+        res += std::max(trans_disjuncts, 1ul);
     }
 
     return res;
@@ -432,57 +434,57 @@ size_t Mata::IntermediateAut::get_number_of_disjuncts() const
  * @param aut Automaton to which transition will be added.
  * @param tokens Series of tokens representing transition formula
  */
-void Mata::IntermediateAut::parse_transition(Mata::IntermediateAut &aut, const std::vector<std::string> &tokens)
+void mata::IntermediateAut::parse_transition(mata::IntermediateAut &aut, const std::vector<std::string>& tokens)
 {
     assert(tokens.size() > 1); // transition formula has at least two items
-    const Mata::FormulaNode lhs = create_node(aut, tokens[0]);
-    const std::vector<std::string> rhs(tokens.begin()+1, tokens.end());
+    mata::FormulaNode lhs = create_node(aut, tokens[0]);
+    std::vector<std::string> rhs(tokens.begin()+1, tokens.end());
 
-    std::vector<Mata::FormulaNode> postfix;
+    std::vector<mata::FormulaNode> postfix;
 
     // add implicit conjunction to NFA explicit states, i.e. p a q -> p a & q
-    if (aut.automaton_type == Mata::IntermediateAut::NFA && tokens[tokens.size()-2] != "&") {
+    if (aut.automaton_type == mata::IntermediateAut::AutomatonType::NFA && tokens[tokens.size() - 2] != "&") {
         // we need to take care about this case manually since user does not need to determine
         // symbol and state naming and put conjunction to transition
-        if (aut.alphabet_type != Mata::IntermediateAut::BITVECTOR) {
+        if (aut.alphabet_type != mata::IntermediateAut::AlphabetType::BITVECTOR) {
             assert(rhs.size() == 2);
-            postfix.push_back(Mata::FormulaNode{Mata::FormulaNode::Type::OPERAND, rhs[0], rhs[0],
-                                                Mata::FormulaNode::OperandType::SYMBOL});
-            postfix.push_back(create_node(aut,rhs[1]));
-        } else if (aut.alphabet_type == Mata::IntermediateAut::BITVECTOR) {
-            // this is a case where rhs state not separated by conjunction from rest of trans
-            postfix = infix_to_postfix(aut, std::vector<std::string>(rhs.begin(), rhs.end()-1));
-            postfix.push_back(create_node(aut,rhs.back()));
+            postfix.emplace_back(mata::FormulaNode::Type::OPERAND, rhs[0], rhs[0], mata::FormulaNode::OperandType::SYMBOL);
+            postfix.emplace_back(create_node(aut, rhs[1]));
+        } else if (aut.alphabet_type == mata::IntermediateAut::AlphabetType::BITVECTOR) {
+            // This is a case where rhs state is not separated by a conjunction from the rest of the transitions.
+            std::string last_token{ rhs.back() };
+            rhs.pop_back();
+            postfix = infix_to_postfix(aut, rhs);
+            postfix.emplace_back(create_node(aut, last_token));
         } else
             assert(false && "Unknown NFA type");
 
-        postfix.push_back(Mata::FormulaNode(
-                Mata::FormulaNode::OPERATOR, "&", "&", Mata::FormulaNode::AND));
+        postfix.emplace_back(mata::FormulaNode::Type::OPERATOR, "&", "&", mata::FormulaNode::OperatorType::AND);
     } else
         postfix = infix_to_postfix(aut, rhs);
 
-    for (const auto& node : postfix) {
+    #ifndef NDEBUG
+    for (const auto& node: postfix) {
         assert(node.is_operator() || (node.name != "!" && node.name != "&" && node.name != "|"));
         assert(node.is_leftpar() || node.name != "(");
         assert(node.is_rightpar() || node.name != ")");
     }
-    const Mata::FormulaGraph graph = postfix_to_graph(postfix);
-
-    aut.transitions.push_back(std::pair<Mata::FormulaNode,Mata::FormulaGraph>(lhs, graph));
+    #endif // #ifndef NDEBUG.
+    aut.transitions.emplace_back(std::move(lhs), postfix_to_graph(std::move(postfix)));
 }
 
-std::unordered_set<std::string> Mata::FormulaGraph::collect_node_names() const
+std::unordered_set<std::string> mata::FormulaGraph::collect_node_names() const
 {
     std::unordered_set<std::string> res;
-    std::vector<const Mata::FormulaGraph *> stack;
+    std::vector<const mata::FormulaGraph *> stack;
 
-    stack.push_back(reinterpret_cast<const FormulaGraph *const>(&(this->node)));
+    stack.emplace_back(reinterpret_cast<const FormulaGraph*>(&(this->node)));
     while (!stack.empty()) {
         const FormulaGraph* g = stack.back();
         assert(g != nullptr);
         stack.pop_back();
 
-        if (g->node.type == FormulaNode::UNKNOWN)
+        if (g->node.type == FormulaNode::Type::UNKNOWN)
            continue; // skip undefined nodes
 
         if (g->node.is_operand()) {
@@ -497,7 +499,7 @@ std::unordered_set<std::string> Mata::FormulaGraph::collect_node_names() const
     return res;
 }
 
-void Mata::FormulaGraph::print_tree(std::ostream& os) const
+void mata::FormulaGraph::print_tree(std::ostream& os) const
 {
     std::vector<const FormulaGraph*> next_level;
     std::vector<const FormulaGraph*> this_level;
@@ -517,12 +519,12 @@ void Mata::FormulaGraph::print_tree(std::ostream& os) const
     }
 }
 
-std::vector<Mata::IntermediateAut> Mata::IntermediateAut::parse_from_mf(const Mata::Parser::Parsed &parsed)
+std::vector<mata::IntermediateAut> mata::IntermediateAut::parse_from_mf(const mata::parser::Parsed &parsed)
 {
-    std::vector<Mata::IntermediateAut> result;
+    std::vector<mata::IntermediateAut> result;
     result.reserve(parsed.size());
 
-    for (const auto& parsed_section : parsed) {
+    for (const parser::ParsedSection& parsed_section: parsed) {
         if (parsed_section.type.find("FA") == std::string::npos) {
             continue;
         }
@@ -532,35 +534,35 @@ std::vector<Mata::IntermediateAut> Mata::IntermediateAut::parse_from_mf(const Ma
     return result;
 }
 
-const Mata::FormulaGraph& Mata::IntermediateAut::get_symbol_part_of_transition(
+const mata::FormulaGraph& mata::IntermediateAut::get_symbol_part_of_transition(
         const std::pair<FormulaNode, FormulaGraph>& trans) const
 {
     if (!this->is_nfa()) {
         throw std::runtime_error("We currently support symbol extraction only for NFA");
     }
-    assert(trans.first.is_operand() && trans.first.operand_type == FormulaNode::STATE);
+    assert(trans.first.is_operand() && trans.first.operand_type == FormulaNode::OperandType::STATE);
     assert(trans.second.node.is_operator()); // conjunction with rhs state
     assert(trans.second.children[1].node.is_operand()); // rhs state
     return trans.second.children[0];
 }
 
-void Mata::IntermediateAut::add_transition(const FormulaNode& lhs, const FormulaNode& symbol, const FormulaGraph& rhs)
+void mata::IntermediateAut::add_transition(const FormulaNode& lhs, const FormulaNode& symbol, const FormulaGraph& rhs)
 {
-    FormulaNode conjunction(FormulaNode::OPERATOR, "&", "&", FormulaNode::AND);
+    FormulaNode conjunction(FormulaNode::Type::OPERATOR, "&", "&", FormulaNode::OperatorType::AND);
     FormulaGraph graph(conjunction);
-    graph.children.push_back(FormulaGraph(symbol));
+    graph.children.emplace_back(symbol);
     graph.children.push_back(rhs);
-    this->transitions.push_back(std::pair<FormulaNode, FormulaGraph>(lhs, graph));
+    this->transitions.emplace_back(lhs, std::move(graph));
 }
 
-void Mata::IntermediateAut::add_transition(const FormulaNode& lhs, const FormulaNode& rhs)
+void mata::IntermediateAut::add_transition(const FormulaNode& lhs, const FormulaNode& rhs)
 {
     assert(rhs.is_operand());
     FormulaGraph graph(rhs);
-    this->transitions.push_back(std::pair<FormulaNode, FormulaGraph>(lhs, graph));
+    this->transitions.emplace_back(lhs, std::move(graph));
 }
 
-void Mata::IntermediateAut::print_transitions_trees(std::ostream& os) const
+void mata::IntermediateAut::print_transitions_trees(std::ostream& os) const
 {
     for (const auto& trans : transitions) {
         os << trans.first.raw << " -> ";
@@ -568,7 +570,7 @@ void Mata::IntermediateAut::print_transitions_trees(std::ostream& os) const
     }
 }
 
-std::unordered_set<std::string> Mata::IntermediateAut::get_positive_finals() const
+std::unordered_set<std::string> mata::IntermediateAut::get_positive_finals() const
 {
     if (!is_graph_conjunction_of_negations(this->final_formula))
         throw (std::runtime_error("Final formula is not a conjunction of negations"));
@@ -587,7 +589,7 @@ std::unordered_set<std::string> Mata::IntermediateAut::get_positive_finals() con
     return all;
 }
 
-bool Mata::IntermediateAut::is_graph_conjunction_of_negations(const Mata::FormulaGraph &graph) {
+bool mata::IntermediateAut::is_graph_conjunction_of_negations(const mata::FormulaGraph &graph) {
     const FormulaGraph *act_graph = &graph;
 
     while (act_graph->children.size() == 2) {
@@ -603,13 +605,13 @@ bool Mata::IntermediateAut::is_graph_conjunction_of_negations(const Mata::Formul
     return (act_graph->node.is_operator() && act_graph->node.is_neg());
 }
 
-std::ostream& std::operator<<(std::ostream& os, const Mata::IntermediateAut& inter_aut)
+std::ostream& std::operator<<(std::ostream& os, const mata::IntermediateAut& inter_aut)
 {
     std::string type = inter_aut.is_nfa() ? "NFA" : (inter_aut.is_afa() ? "AFA" : "Unknown");
     os << "Intermediate automaton type " << type << '\n';
-    os << "Naming - state: " << inter_aut.state_naming << " symbol: " << inter_aut.symbol_naming << " node: "
-        << inter_aut.node_naming << '\n';
-    os << "Alphabet " << inter_aut.alphabet_type << '\n';
+    os << "Naming - state: " << static_cast<size_t>(inter_aut.state_naming) << " symbol: "
+       << static_cast<size_t>(inter_aut.symbol_naming) << " node: " << static_cast<size_t>(inter_aut.node_naming) << '\n';
+    os << "Alphabet " << static_cast<size_t>(inter_aut.alphabet_type) << '\n';
 
     os << "Initial states: ";
     for (const auto& state : inter_aut.initial_formula.collect_node_names()) {
